@@ -1,11 +1,9 @@
 {-# LANGUAGE CPP, Rank2Types, MagicHash, UnboxedTuples, ExistentialQuantification #-}
+{-# OPTIONS_GHC -fno-warn-unused-binds #-} -- SPEC2
 
-{- |
-    Internal module, do not import or use.
--}
-
+-- | Internal module, do not import or use.
 module Data.Generics.Uniplate.Internal.Utils(
-    unsafeCoerce, builder, unsafePerformIO, inlinePerformIO, concatCont
+    unsafeCoerce, builder, unsafePerformIO, inlinePerformIO, concatCont, SPEC(SPEC)
     ) where
 
 #if __GLASGOW_HASKELL__ >= 702
@@ -24,6 +22,11 @@ import GHC.IO(IO(IO))
 #endif
 #endif
 
+#if __GLASGOW_HASKELL__ >= 701
+import GHC.Exts(SpecConstrAnnotation(..))
+{-# ANN type SPEC ForceSpecConstr #-}
+#endif
+
 
 {-# INLINE builder #-}
 -- | GHCs @foldr@\/@build@ system, but on all platforms
@@ -50,3 +53,7 @@ inlinePerformIO = unsafePerformIO
 -- | Perform concatentation of continuations
 concatCont :: [a -> a] -> a -> a
 concatCont xs rest = foldr ($) rest xs
+
+
+-- | Constructor specialisation on newer GHC
+data SPEC = SPEC | SPEC2
